@@ -13,14 +13,14 @@ A container for functions for the ack program.
 
 =head1 VERSION
 
-Version 2.999_06
+Version 2.999_07
 
 =cut
 
 our $VERSION;
 our $COPYRIGHT;
 BEGIN {
-    $VERSION = '2.999_06';
+    $VERSION = '2.999_07';
     $COPYRIGHT = 'Copyright 2005-2019 Andy Lester.';
 }
 our $STANDALONE = 0;
@@ -214,10 +214,6 @@ Dumps the help page to the user.
 =cut
 
 sub show_help {
-    my $help_arg = shift || 0;
-
-    return show_help_types() if $help_arg =~ /^types?/;
-
     my $manual_options;
     if ( $App::Ack::STANDALONE ) {
         $manual_options = <<'END';
@@ -252,7 +248,7 @@ Searching:
   -S, --[no]smart-case          Ignore case distinctions in PATTERN,
                                 only if PATTERN contains no upper case.
                                 Ignored if -i or -I are specified.
-  -I                            Turns on case-sensitivity in PATTERN.
+  -I, --no-ignore-case          Turns on case-sensitivity in PATTERN.
                                 Negates -i and --smart-case.
   -v, --invert-match            Invert match: select non-matching lines
   -w, --word-regexp             Force PATTERN to match only whole words
@@ -312,6 +308,8 @@ File presentation:
   --color-colno=COLOR
   --color-lineno=COLOR          Set the color for filenames, matches, line and
                                 column numbers.
+  --help-colors                 Show a list of possible color combinations.
+  --help-rgb-colors             Show a list of advanced RGB colors.
   --flush                       Flush output immediately, even when ack is used
                                 non-interactively (when output goes to a pipe or
                                 file).
@@ -362,8 +360,11 @@ Miscellaneous:
                                 and where they're defined.
   --[no]filter                  Force ack to treat standard input as a pipe
                                 (--filter) or tty (--nofilter)
-  --help, -?                    This help
+  --help                        This help
 $manual_options
+  --help-types                  Display all known types, and how they're defined.
+  --help-colors                 Show a list of possible color combinations.
+  --help-rgb-colors             Show a list of advanced RGB colors.
   --thpppt                      Bill the Cat
   --bar                         The warning admiral
   --cathy                       Chocolate! Chocolate! Chocolate!
@@ -428,6 +429,147 @@ END_OF_HELP
 }
 
 
+=head2 show_help_colors()
+
+Display the colors help subpage.
+
+=cut
+
+sub show_help_colors {
+    App::Ack::print( <<'END_OF_HELP' );
+ack allows customization of the colors it uses when presenting matches
+onscreen.  See the "ACK COLORS" section of the ack manual (ack --man).
+
+Here is a chart of how various color combinations appear: Each of the eight
+foreground colors, on each of the eight background colors or no background
+color, with and without the bold modifier.
+
+Run ack --help-rgb-colors for a chart of the RGB colors.
+
+END_OF_HELP
+
+    _show_color_grid();
+
+    return;
+}
+
+
+=head2 show_help_rgb()
+
+Display the RGB help subpage.
+
+=cut
+
+sub show_help_rgb {
+    App::Ack::print( <<'END_OF_HELP' );
+ack allows customization of the colors it uses when presenting matches
+onscreen.  See the "ACK COLORS" section of the ack manual (ack --man).
+
+Colors may be specified as "rggNNN" where "NNN" is a triplet of digits
+from 0 to 5 specifying the intensity of red, green and blue, respectively.
+
+Here is a grid of the 216 possible values for NNN.
+
+END_OF_HELP
+
+    _show_rgb_grid();
+
+    App::Ack::say( 'Here are the 216 possible colors with the "reverse" modifier applied.', "\n" );
+
+    _show_rgb_grid( 'reverse' );
+
+    return;
+}
+
+
+sub _show_color_grid {
+    my $cell_width = 7;
+
+    my @fg_colors = qw( black red green yellow blue magenta cyan white );
+    my @bg_colors = map { "on_$_" } @fg_colors;
+
+    App::Ack::say(
+        _color_cell( '' ),
+        map { _color_cell( $_ ) } @fg_colors
+    );
+
+    App::Ack::say(
+        _color_cell( '' ),
+        map { _color_cell( '-' x $cell_width ) } @fg_colors
+    );
+
+    for my $bg ( '', @bg_colors ) {
+        App::Ack::say(
+            _color_cell( '' ),
+            ( map { _color_cell( $_, "$_ $bg" ) } @fg_colors ),
+            $bg
+        );
+
+        App::Ack::say(
+            _color_cell( 'bold' ),
+            ( map { _color_cell( $_, "bold $_ $bg" ) } @fg_colors ),
+            $bg
+        );
+        App::Ack::say();
+    }
+
+    return;
+}
+
+
+sub _color_cell {
+    my $text  = shift;
+    my $color = shift;
+
+    my $cell_width = 7;
+    $text = sprintf( '%-*s', $cell_width, $text );
+
+    return ($color ? Term::ANSIColor::colored( $text, $color ) : $text) . ' ';
+}
+
+
+sub _show_rgb_grid {
+    my $modifier = shift // '';
+
+    my $grid = <<'HERE';
+544 544 544 544 544 554 554 554 554 554 454 454 454 454 454 455 455 455 455 455 445 445 445 445 445 545 545 545 545 545
+533 533 533 543 543 553 553 553 453 453 353 353 353 354 354 355 355 355 345 345 335 335 335 435 435 535 535 535 534 534
+511 521 531 531 541 551 451 451 351 251 151 152 152 153 154 155 145 145 135 125 115 215 215 315 415 515 514 514 513 512
+500 510 520 530 540 550 450 350 250 150 050 051 052 053 054 055 045 035 025 015 005 105 205 305 405 505 504 503 502 501
+400 410 410 420 430 440 340 340 240 140 040 041 041 042 043 044 034 034 024 014 004 104 104 204 304 404 403 403 402 401
+300 300 310 320 320 330 330 230 130 130 030 030 031 032 032 033 033 023 013 013 003 003 103 203 203 303 303 302 301 301
+200 200 200 210 210 220 220 220 120 120 020 020 020 021 021 022 022 022 012 012 002 002 002 102 102 202 202 202 201 201
+100 100 100 100 100 110 110 110 110 110 010 010 010 010 010 011 011 011 011 011 001 001 001 001 001 101 101 101 101 101
+
+522 522 532 542 542 552 552 452 352 352 252 252 253 254 254 255 255 245 235 235 225 225 325 425 425 525 525 524 523 523
+
+411 411 421 431 431 441 441 341 241 241 141 141 142 143 143 144 144 134 124 124 114 114 214 314 314 414 414 413 412 412
+
+422 422 432 432 432 442 442 442 342 342 242 242 242 243 243 244 244 244 234 234 224 224 224 324 324 424 424 424 423 423
+
+311 311 311 321 321 331 331 331 231 231 131 131 131 132 132 133 133 133 123 123 113 113 113 213 213 313 313 313 312 312
+
+433 433 433 433 433 443 443 443 443 443 343 343 343 343 343 344 344 344 344 344 334 334 334 334 334 434 434 434 434 434
+211 211 211 211 211 221 221 221 221 221 121 121 121 121 121 122 122 122 122 122 112 112 112 112 112 212 212 212 212 212
+
+322 322 322 322 322 332 332 332 332 332 232 232 232 232 232 233 233 233 233 233 223 223 223 223 223 323 323 323 323 323
+
+555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555 555
+444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444 444
+333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333 333
+222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222 222
+111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111 111
+000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000
+HERE
+
+    $grid =~ s/(\d\d\d)/Term::ANSIColor::colored( "$1", "$modifier rgb$1" )/eg;
+
+    App::Ack::say( $grid );
+
+    return;
+}
+
+
 sub show_docs {
     my $section = shift;
 
