@@ -732,6 +732,8 @@ sub get_expected_options {
         '--heading',
         '--help',
         '--help-types',
+        '--help-colors',
+        '--help-rgb-colors',
         '--ignore-ack-defaults',
         '--ignore-case',
         '--ignore-dir',
@@ -774,7 +776,6 @@ sub get_expected_options {
         '--with-filename',
         '--word-regexp',
         '-1',
-        '-?',
         '-A',
         '-B',
         '-C',
